#include "../include/FileManager.h"

#include "../include/Config.h"

FileManager::FileManager() : fileDirectory(Config::getValue("filedirectory")) {}

FileManager::~FileManager() {
  cancelPut();
  closeGetFile();
}

bool FileManager::isUploading() { return this->putFile.is_open(); }

bool FileManager::isDownloading() { return this->getFile.is_open(); }

bool FileManager::openPutFile(const std::string &filename) {
  this->putBaseFileName = filename;
  this->putFileName = this->fileDirectory;
  this->putFileName.append(filename);

  std::ifstream ifile(this->putFileName);
  if (ifile.is_open()) {
    // file alread exists
    ifile.close();
    closePutFile();
    return false;
  }

  // open file and test if it already exists
  this->putFile.open(this->putFileName, std::ios::app | std::ios::binary);
  return true;
}

bool FileManager::openGetFile(const std::string &filename, int &chunks) {
  this->getBaseFileName = filename;
  std::string file = this->fileDirectory;
  file.append(filename);

  this->getFile.open(file, std::ios::ate | std::ios::binary);

  if (this->getFile.is_open() == 0) {
    // file does not exist or cannot be opened
    return false;
  } else {
    this->getFileSize = this->getFile.tellg();
    chunks = this->getFileSize / max_data_length +
             (this->getFileSize % max_data_length == 0 ? 0 : 1);

    this->getFile.seekg(std::ios::beg);
    return true;
  }
}

void FileManager::closePutFile() { this->putFile.close(); }

void FileManager::closeGetFile() { this->getFile.close(); }

void FileManager::cancelPut() {
  if (isUploading()) {
    closePutFile();
    std::remove(this->putFileName.c_str());
  }
}

bool FileManager::checkFilename(const std::string &name) {
  return name.find('/') == std::string::npos;
}

std::string FileManager::getGetBaseFileName() { return this->getBaseFileName; }

std::string FileManager::getPutBaseFileName() { return this->putBaseFileName; }

void FileManager::writePut(const std::vector<char> &data) {
  this->putFile.write(data.data(), data.size());
}

std::vector<char> FileManager::readGet() {
  char fileBuffer[max_data_length];

  // store position before read
  std::streamoff read = this->getFile.tellg();
  if (read + max_data_length > this->getFileSize) {
    read = this->getFileSize % max_data_length;
  } else {
    read = max_data_length;
  }

  this->getFile.read(fileBuffer, read);

  std::vector<char> data;
  data.assign(fileBuffer, fileBuffer + read);

  return data;
}
